function MC_tournoi_stratB (N)
% La fonction 'MC_tournoi' évalue la probabilité de victoire de #16 dans le
% tournoi de tennis, en appliquant la technique de stratification  avec
% échantillonnage optimisé. 'N' est le nombre minimal de simulations
% demandées.

% 'taillestrates' est la taille exacte de chaque strate, calculée
% théoriquement.
taillestrates = [91, 52, 28, 14, 10] / 195;

% On alloue de la mémoire pour 'nombredesimulations', 'succes',
% 'tempsparsimulation' et 'efficacite', qui contiendront respectivement le
% nombre de simulations, le nombre de simulations victorieuses, le temps
% par simulation et l'efficacité pour chaque estimateur de la forme
% E_i(taillestrates(i)*succes) (voir plus loin).
nombredesimulations = zeros (1, 5);
succes = zeros (1, 5);
tempsparsimulation = zeros (1, 5);
efficacite = zeros (1, 5);

% Notre espérance se ramène à la somme des E_i(taillestrates(i)*succes), où
% E_i est la loi uniforme sur la strate i. Nous devons évaluer l'efficacité
% de l'estimateur de chacune de ces espérances. Pour ce faire, nous lançons
% un certain nombre de simulations de E_i jusqu'à avoir un nombre
% raisonnable de succès (disons 8 — cela donne des calculs préliminaires un
% peu lents, mais en dessous l'évaluation de l'effiacité serait trop
% instable pour donner de bons résultats), afin d'estimer l'efficacité de
% l'estimateur.
for i = 1:5
    % 'succesi' compte le nombre de succès.
    succesi = 0;
    % 'Ni' compte le nombre de simulations, qui n'est pas connu à l'avance
    % ici.
    Ni = 0;
    % On lance un chronomètre. J'utilise ici la syntaxe développé de 'tic'
    % au cas où tout cela s'insèrerait dans un processus de chronométrage
    % global : le temps du chronomètre pour ce morceau de simulation est
    % référé par l'identité 'tstarti'.
    tstarti = tic;
    while (succesi < 8)
        succesi = succesi + seizegagne_stratB (i);
        Ni = Ni + 1;
    end
    % On arrête le chronomètre ; 'ti' est le temps écoulé pour ce morceau
    % de simulation.
    ti = toc (tstarti);
    % On remplit 'nombredesimulations', 'succes' et 'tempsparsimulation' en
    % fonction des calculs qu'on vient de faire.
    nombredesimulations(i) = Ni;
    succes(i) = succesi;
    tempsparsimulation(i) = ti / Ni;
    % Quelle est l'efficacité (approximative) de notre estimateur ? La
    % variance par simulation vaut taillestrates(i)^2*pi_*(1-pi_)
    % (n'oublions pas qu'on a multiplié par taillestrates(i) à la fin !),
    % où pi_ est l'estimateur de la probabilité de succès (soit succes/Ni),
    % et le temps par simulation vient d'être calculé. Ici c'est important
    % de prendre l'efficacité tout court et par juste l'efficacité par
    % estimateur : en effet, il y a lieu de penser que le temps de calcul
    % par simulation ne sera pas le même d'une strate à l'autre, vu que
    % quand #16 a un tableau plus facile il est susceptible d'aller plus
    % loin dans le tournoi et donc il faut plus de calculs avant de
    % connaite son éventuel succes.
    pi_ = succesi / Ni;
    efficacite(i) = 1 / ...
        (taillestrates(i)^2 * pi_ * (1 - pi_) * tempsparsimulation(i));
    % On affiche quelques informations
    fprintf ('POUR INFORMATION - Pour la strate #%d :\n', i);
    fprintf ('* La probabilité de succès est d''environ %.2e ;\n', pi_);
    fprintf ('Le temps de calcul par simulation est d''environ %.2e s ;\n', tempsparsimulation(i));
    fprintf ('L''efficacité (variance inverse par unité de temps) est d''environ %.2e.\n', efficacite(i));
    fprintf ('Note - Ces calculs préliminaires m''ont demandé %.2e s ', ti);
    fprintf ('et m''ont conduit à effectuer %d simulations.\n\n', Ni);
end

% À ce stade, nous pouvons calculer la façon intelligente d'allouer le
% calcul entre les différentes strates : le cout (en l'occurrence, le
% temps) consacré à chaque strate doit être proportionnel à son efficacité
% à la puissance (-1/2), ce que nous convertissons ensuite en nombre de
% simulations grâce à tempsparsimulation. 'Nsimulationsparstrate'
% contiendra, d'abord à constante près, puis exactement, le nombre de
% simulations à effectuer par strate.
Nsimulationsparstrate = 1 ./ sqrt (efficacite) ./ tempsparsimulation;
% On était pour l'instant à constante près ; il faut que le total vaille N.
Nsimulationsparstrate = N * Nsimulationsparstrate / sum (Nsimulationsparstrate);
% On arrondit à la partie supérieure, vu qu'on veut des entiers.
Nsimulationsparstrate = ceil (Nsimulationsparstrate);
% En fait, il serait idiot de jeter les simulations déjà faites, si leur
% nombre excède celui requis ; d'où la ligne suivante.
Nsimulationsparstrate = max (Nsimulationsparstrate, nombredesimulations);
% On affiche quelques informations
fprintf ('POUR INFORMATION - Je vais répartir mes simulations entre strates de la façon suivante :\n');
for i = 1:5
    fprintf ('* Strate #%d : %d simulations.\n', i, Nsimulationsparstrate(i));
end
fprintf ('\n');

% 'estimateurspartiels' contiendra les estimateurs de chaque
% E_i(taillestrates(i)*succes) ; 'variancespartielles' contiendra les
% variances /des estimateurs/.
estimateurspartiels = zeros (1, 5);
variancespartielles = zeros (1, 5);

% Nous sommes prêts à lancer maintenant les boucles de simulations
% principales ! Celles-ci ressemblent bien sûr beaucoup aux premières
% boucles lancées plus haut ; je ne commente que les différences.
for i = 1:5
    % On récupère les simulations déjà obtenues.
    succesi = succes(i);
    % 'n' est le numéro de la simulation en cours. Attention, on na part
    % pas de 1 car on a déjà fait des simulations avant !
    for n = nombredesimulations(i)+1:Nsimulationsparstrate(i)
        succesi = succesi + seizegagne_stratB (i);
    end
    pi_ = succesi / Nsimulationsparstrate(i);
    estimateurspartiels(i) = taillestrates(i) * pi_;
    % Ne pas oublier de diviser par le nombre de simulations quand on
    % cherche la variance de l'estimateur !
    variancespartielles(i) = taillestrates(i)^2 * pi_ * (1 - pi_) / Nsimulationsparstrate(i);
end

% L'estimateur global 'P' est obtenu comme la somme des estimateurs
% partiels.
P = sum (estimateurspartiels);
% La variance globale /de l'estimateur/, notée 'VAR_EST', est la somme des
% variances partielles, vu que les différents estimateurs partiels sont
% indépendants.
VAR_EST = sum (variancespartielles);
% Il n'y a plus qu'à calculer la marge d'erreur associée à VAR_EST et à
% afficher le résultat.
fprintf ...
    ('ESTIMATION DE LA PROBABILITÉ DE SUCCÈS DE #16 : %.7f ± %.7f\n', ...
    P, 1.96*sqrt(VAR_EST));

return
end